@@ -25,6 +25,8 @@
 #include "pic.h"
 #include "fpu.h"
 
+extern bool do_lds_wraparound;
+
 bool CPU_RDMSR();
 bool CPU_WRMSR();
 bool CPU_SYSENTER();
@@ -45,6 +47,8 @@ static inline bool _seg_limit_check(void) {
 
 #define DoString DoString_Normal286
 
+static uint16_t last_ea86_offset;
+
 extern bool ignore_opcode_63;
 
 #if C_DEBUG
@@ -179,6 +183,7 @@ Bits CPU_Core286_Normal_Run(void) {
 		LOADIP;
 		core.prefixes=0;
 		core.opcode_index=0;
+		last_ea86_offset=0;
 		core.ea_table=&EATable[0];
 		BaseDS=SegBase(ds);
 		BaseSS=SegBase(ss);
