@@ -650,7 +650,6 @@ private:
 			cur_col=0;
 			break;
 		case '\n':
-			cur_col=0;
 			cur_row++;
 			break;
 		case '\t':
@@ -1050,6 +1049,10 @@ std::string log_dev_con_str;
 bool logging_con = false;
 bool DOS_BreakTest(bool print);
 void DOS_BreakAction();
+bool read_kanji1 = false;
+uint8_t temp_char = 0;
+void WriteChar(uint16_t col, uint16_t row, uint8_t page, uint16_t chr, uint8_t attr, bool useattr);
+extern bool dbcs_sbcs;
 
 bool device_CON::Write(const uint8_t * data,uint16_t * size) {
 	uint16_t count=0;
@@ -1079,7 +1082,7 @@ bool device_CON::Write(const uint8_t * data,uint16_t * size) {
 		return true;
 	}
     Bitu i;
-    uint8_t col,page;
+    uint8_t col,row,page;
 
     INT10_SetCurMode();
 
@@ -1110,6 +1113,42 @@ bool device_CON::Write(const uint8_t * data,uint16_t * size) {
                 log_dev_con_str += (char)data[count];
         }
 
+        page = real_readb(BIOSMEM_SEG, BIOSMEM_CURRENT_PAGE);
+        col = CURSOR_POS_COL(page);
+        BIOS_NCOLS;
+
+        if(isDBCSCP() && !dos.direct_output
+#if defined(USE_TTF)
+            && dbcs_sbcs
+#endif
+            ) { // Consideration of first byte of DBCS characters at the end of line 
+            if(!read_kanji1 && isKanji1(data[count])) {
+                read_kanji1 = true;
+                temp_char = data[count];
+                count++;
+                continue;
+            }
+            else if(read_kanji1) {
+                if(col == ncols - 1 && isKanji2(data[count])) {
+                    BIOS_NROWS;
+                    row = CURSOR_POS_ROW(page);
+                    WriteChar(col, row, page, ' ', ansi.attr, true);
+                    if(nrows == row + 1) {
+                        INT10_ScrollWindow(0, 0, (uint8_t)(nrows - 1), (uint8_t)(ncols - 1), -1, ansi.attr, page);
+                        INT10_SetCursorPos(row, 0, page);
+                    }
+                    else INT10_SetCursorPos(row + 1, 0, page);
+                    Output(temp_char);
+                    Output(data[count]);
+                    count++;
+                    read_kanji1 = false;
+                    continue;
+                }
+                Output(temp_char);
+                read_kanji1 = false;
+            }
+        }
+
         if (!ansi.esc){
             if(data[count]=='\033' && ansi.installed) {
                 /*clear the datastructure */
@@ -1120,7 +1159,6 @@ bool device_CON::Write(const uint8_t * data,uint16_t * size) {
                 continue;
             } else if(data[count] == '\t' && !dos.direct_output) {
                 /* expand tab if not direct output */
-                page = real_readb(BIOSMEM_SEG,BIOSMEM_CURRENT_PAGE);
                 do {
                     Output(' ');
                     col=CURSOR_POS_COL(page);
@@ -1128,16 +1166,12 @@ bool device_CON::Write(const uint8_t * data,uint16_t * size) {
                 count++;
                 continue;
             } else if (data[count] == 0x1A && IS_PC98_ARCH) {
-                page = real_readb(BIOSMEM_SEG,BIOSMEM_CURRENT_PAGE);
-
                 /* it also redraws the function key row */
                 update_pc98_function_row(pc98_function_row_mode,true);
 
                 INT10_ScrollWindow(0,0,255,255,0,ansi.attr,page);
                 Real_INT10_SetCursorPos(0,0,page);
             } else if (data[count] == 0x1E && IS_PC98_ARCH) {
-                page = real_readb(BIOSMEM_SEG,BIOSMEM_CURRENT_PAGE);
-
                 Real_INT10_SetCursorPos(0,0,page);
             } else { 
                 Output(data[count]);
@@ -1631,7 +1665,7 @@ void device_CON::Output(uint8_t chr) {
 			uint8_t page=real_readb(BIOSMEM_SEG,BIOSMEM_CURRENT_PAGE);
 			uint8_t col=CURSOR_POS_COL(page);
 			uint8_t row=CURSOR_POS_ROW(page);
-			BIOS_NCOLS;BIOS_NROWS;
+ 			BIOS_NCOLS;BIOS_NROWS;
 			if (nrows==row+1 && (chr=='\n' || (ncols==col+1 && chr!='\r' && chr!=8 && chr!=7))) {
 				INT10_ScrollWindow(0,0,(uint8_t)(nrows-1),(uint8_t)(ncols-1),-1,ansi.attr,page);
 				INT10_SetCursorPos(row-1,col,page);
