@@ -298,10 +298,12 @@ void Set_Label(char const * const input, char * const output, bool cdrom) {
     strncpy(upcasebuf, input, 11);
     //DBCS_upcase(upcasebuf);  /* Another mscdex quirk. Label is not always uppercase. (Daggerfall) */ 
 
-    (void)cdrom;
-
     while (togo > 0) {
         if(upcasebuf[vnamePos] == 0) str_end = true;
+        else if(cdrom && vnamePos == 8 && !str_end && upcasebuf[vnamePos] != '.') {
+            output[labelPos] = '.'; // add a dot between 8th and 9th character (Descent 2 installer needs this)
+            labelPos++;
+        }
         output[labelPos] = !str_end ? upcasebuf[vnamePos] : 0x0; // Pad empty characters with 0x00
         labelPos++;
         vnamePos++;
