@@ -155,11 +155,21 @@ void write_p3d5_et4k(Bitu reg,Bitu val,Bitu iolen) {
         break;
 
     // 3d4h index 36h - Video System Configuration 1 (R/W)
-    // VGADOC provides a lot of info on this register, Ferraro has significantly less detail.
-    // This is unlikely to be used by any games. Bit 4 switches chipset into linear mode -
-    // that may be useful in some cases if there is any software actually using it.
-    // TODO (not near future): support linear addressing
-    STORE_ET4K(3d4, 36);
+    // Bits 0-2: Refresh count per line - 1
+    // Bit 3: Font width control (1=up to 16-bit, 0=8-bit VGA compatible font)
+    // Bit 4: Segment/linear system configuration (0=segment 1=linear),
+    //        meaning to present the 1MB of the card as a linear framebuffer somewhere on the SSA bus
+    // Bit 5: Addressing mode (0=IBM 1=TLI).
+    //        If set, enables contiguous address mapping which is a much more efficient use of the card's resources.
+    //        If not set, address mapping is compatible with VGA.
+    // Bit 6: 16-bit display memory read/write (1=enable)
+    // Bit 7: 16-bit I/O read/write (1=enable)
+    case 0x36:
+        if (val != et4k.store_3d4_36) {
+            et4k.store_3d4_36 = val;
+            // TODO
+        }
+        break;
 
     // 3d4h index 37 - Video System Configuration 2 (R/W)
     // Bits 0,1, and 3 provides information about memory size:
@@ -274,13 +284,18 @@ Bitu read_p3cd_et4k(Bitu port,Bitu iolen) {
 void write_p3c0_et4k(Bitu reg,Bitu val,Bitu iolen) {
     (void)iolen;//UNUSED
     switch(reg) {
-    // 3c0 index 16h: ATC Miscellaneous
-    // VGADOC provides a lot of information, Ferarro documents only two bits
-    // and even those incompletely. The register is used as part of identification
-    // scheme.
-    // Unlikely to be used by any games but double timing may be useful.
-    // TODO: Figure out if this has any practical use
-    STORE_ET4K(3c0, 16);
+    /* 3c0 index 16h: ATC Miscellaneous
+       bit 7: bypass the internal palette
+       bit 6: 2-byte character code (ET4000 Rev E) [TODO: See addendum 6.1]
+       bit 4-5: Select high resolution/color mode
+                00b: Normal
+                01b: Reserved
+                10b: High resolution mode (up to 256 colors)
+                11b: High-color 16-bit/pixel
+       bit 0-3: Reserved */
+        case 0x16:
+            et4k.store_3c0_16 = val;
+            break;
     /*
     3C0h index 17h (R/W):  Miscellaneous 1
     bit   7  If set protects the internal palette ram and redefines the attribute
@@ -443,9 +458,9 @@ void DetermineMode_ET4K() {
                 VGA_SetMode(M_LIN8);
             }
         }
-        else if (vga.gfx.mode & 0x20) VGA_SetMode(M_CGA4);
-        else if ((vga.gfx.miscellaneous & 0x0c)==0x0c) VGA_SetMode(M_CGA2);
-        else VGA_SetMode((et4k.biosMode<=0x13)?M_EGA:M_LIN4);
+        else {
+            VGA_SetMode((et4k.biosMode<=0x13)?M_EGA:M_LIN4);
+        }
     } else {
         VGA_SetMode(M_TEXT);
     }
@@ -627,8 +642,8 @@ void SVGA_Setup_TsengET4K(void) {
     // From the depths of X86Config, probably inexact
     VGA_SetClock(0,CLK_25);
     VGA_SetClock(1,CLK_28);
-    VGA_SetClock(2,32400);
-    VGA_SetClock(3,35900);
+    VGA_SetClock(2,32515);
+    VGA_SetClock(3,40000);
     VGA_SetClock(4,39900);
     VGA_SetClock(5,44700);
     VGA_SetClock(6,31400);
@@ -958,8 +973,6 @@ void DetermineMode_ET3K() {
     // merge them.
     if (vga.attr.mode_control & 1) {
         if (vga.gfx.mode & 0x40) VGA_SetMode((et3k.biosMode<=0x13)?M_VGA:M_LIN8); // Ugly...
-        else if (vga.gfx.mode & 0x20) VGA_SetMode(M_CGA4);
-        else if ((vga.gfx.miscellaneous & 0x0c)==0x0c) VGA_SetMode(M_CGA2);
         else VGA_SetMode((et3k.biosMode<=0x13)?M_EGA:M_LIN4);
     } else {
         VGA_SetMode(M_TEXT);
