@@ -1331,8 +1331,6 @@ void voodoo_ogl_texture_clear(UINT32 texbase, int TMU) {
 }
 
 void voodoo_ogl_draw_pixel(int x, int y, bool has_rgb, bool has_alpha, int r, int g, int b, int a) {
-	GLfloat x2, y2;
-
 	if (m_hProgramObject != 0) {
 		glUseProgramObjectARB(0);
 		m_hProgramObject = 0;
@@ -1352,9 +1350,6 @@ void voodoo_ogl_draw_pixel(int x, int y, bool has_rgb, bool has_alpha, int r, in
 
 	VOGL_SetAlphaMode(0, 0,0,0,0);
 
-	x2 = (GLfloat) x + 0.5;
-	y2 = (GLfloat) y + 0.5;
-
 	VOGL_BeginMode(GL_POINTS);
 	glColor4ub((GLubyte)(r&0xff), (GLubyte)(g&0xff), (GLubyte)(b&0xff), (GLubyte)(a&0xff));
 	float dx = (float)new_width / (float)v->fbi.width;
@@ -1416,7 +1411,6 @@ void voodoo_ogl_draw_z(int x, int y, int z) {
 
 void voodoo_ogl_draw_pixel_pipeline(int x, int y, int r, int g, int b) {
 //	VOGL_ClearBeginMode();
-	GLfloat x2, y2;
 
 	// TODO redo everything //
 	if (m_hProgramObject != 0) {
@@ -1448,9 +1442,6 @@ void voodoo_ogl_draw_pixel_pipeline(int x, int y, int r, int g, int b) {
 		VOGL_SetAlphaMode(0, 0,0,0,0);
 	}
 
-	x2 = (GLfloat) x + 0.5;
-	y2 = (GLfloat) y + 0.5;
-
 	VOGL_BeginMode(GL_POINTS);
 //	glBegin(GL_POINTS);
 //	glColor3f((float)r/255.0f, (float)g/255.0f, (float)b/255.0f);
@@ -1799,6 +1790,7 @@ void voodoo_ogl_reset_videomode(void) {
         ShowWindow(GetHWND(), SW_MAXIMIZE);
 #endif
     }
+    ApplyPreventCap();
 
     v->ogl_dimchange = true;
 
